==> masamune_firebase-0.2.0.tar.gz: lib/functions/functionsdocument.dart <==
part of masamune.firebase;

/// Task class for executing Firebase Functions functions.
/// 
/// Execute the task with the [call()] method and wait for it to complete with await.
/// 
/// The function must be one layer of Map data.
/// 
/// Parse Map data and use it as a document.
class FunctionsDocument extends TaskDocument<DataField>
  implements ITask, IDataDocument<DataField> {
  static RegExp _regExp = RegExp( "^[a-zA-Z0-9]+-[a-zA-Z0-9]+" );
  /// Create a Completer that matches the class.
  /// 
  /// Do not use from external class.
  @override
  @protected
  Completer createCompleter() => Completer<FunctionsDocument>();
  /// Process to create a new instance.
  /// 
  /// Do not use from outside the class.
  /// 
  /// [path]: Destination path.
  /// [isTemporary]: True if the data is temporary.
  @override
  @protected
  T createCloneInstance<T extends IClonable>( String path, bool isTemporary ) => FunctionsDocument._(
    path: path,
    isTemporary: isTemporary,
    group: this.group,
    order: this.order
  ) as T;
  Firebase get _app {
    if( this.__app == null ) this.__app = Firebase._find(this.protocol);
    return this.__app;
  }
  Firebase __app;
  FirestoreAuth get _auth {
    if( this.__auth == null ) this.__auth = FirestoreAuth._find(this.protocol);
    return this.__auth;
  }
  FirestoreAuth __auth;
  FirestoreFunctions get _functions {
    if( this.__functions == null ){
      this.__functions = FirestoreFunctions._find(this.protocol, this.region);
    }
    return this.__functions;
  }
  FirestoreFunctions __functions;
  HttpsCallable get _callable {
    if( this.__callable == null ){
      this.__callable = this._functions?._functions?.getHttpsCallable(
        functionName: this.rawPath.file
      );
    }
    return this.__callable;
  }
  HttpsCallable __callable;

  /// Task class for executing Firebase Functions functions.
  /// 
  /// Execute the task with the [call()] method and wait for it to complete with await.
  /// 
  /// The function must be one layer of Map data.
  /// 
  /// Parse Map data and use it as a document.
  /// 
  /// [path]: Specify the execution function of Functions in URL format (https://～).
  /// [defaultData]: Initial value when the function is not executed.
  /// [postData]: Parameters to pass to the function.
  /// [timeout]: Timeout time.
  static Future<FunctionsDocument> call(
    String path,
    {
      Map<String, dynamic> defaultValue,
      Map<String,dynamic> data = const {},
      Duration timeout = Const.timeout,
      OrderBy orderBy = OrderBy.none,
      OrderBy thenBy = OrderBy.none,
      String orderByKey,
      String thenByKey
    }
  ){
    path = path?.trimString( Const.slash )
      ?.replaceAll( "//", "/" )
      ?.replaceAll(":/", "://")
      ?.replaceAll("https", "firestore")
      ?.applyTags();
    if( isEmpty(path) ) {
      Log.error( "Path is invalid." );
      return null;
    }
    FunctionsDocument document = PathMap.get<FunctionsDocument>( path );
    if( document != null ) {
      return document.recall(defaultValue: defaultValue, data:data, timeout:timeout);
    }
    document = FunctionsDocument._( path:path );
    if( defaultValue != null ){
      defaultValue?.forEach(
        (key, value) => document.setInternal( DataField( key, value ) )
      );
    }
    document._call(data, timeout);
    return document.future;
  }
  /// Run the function again.
  /// 
  /// [defaultData]: Initial value when the function is not executed.
  /// [postData]: Parameters to pass to the function.
  /// [timeout]: Timeout time.
  Future<FunctionsDocument> recall(
    {
      Map<String, dynamic> defaultValue,
      Map<String,dynamic> data = const {},
      Duration timeout = Const.timeout
    }
  ){
    this.init();
    if( defaultValue != null ){
      this.clear();
      if( defaultValue != null ){
        defaultValue?.forEach(
          (key, value) => this.setInternal( DataField( key, value ) )
        );
      }
    }
    this._call(data, timeout);
    return this.future;
  }
  void _call(Map<String,dynamic> data, Duration timeout) async {
    try {
      if( this._app == null ) this.__app = await Firebase.initialize();
      if( this._auth == null ) this.__auth = await FirestoreAuth.signIn(protocol:this.protocol);
      if( this._functions == null ){
        this.__functions = await FirestoreFunctions.configure(
          protocol: this.protocol,
          region: this.region
        );
      }  
      if( this._callable == null ){
        this.__callable = this._functions._functions
          .getHttpsCallable(functionName: this.rawPath.file);
      }
      Map<dynamic,dynamic> res;
      if( data == null || data.length <= 0 ){
        res = ( await this._callable.call( ).timeout(timeout) )?.data;
      } else {
        res = ( await this._callable.call( data ).timeout(timeout) )?.data;
      }
      if( res == null || res.length <= 0 ){
        this.done();
        return;
      }
      this._build(res)?.forEach((val) => this.setInternal( val ));
      this.done();
    } on TimeoutException catch(e){
      this.timeout(e.toString());
    } catch( e ) {
      this.error( e.toString() );
    }
  }
  Iterable<DataField> _build( Map<dynamic, dynamic> data ) {
    List<DataField> list = ListPool.get();
    data?.forEach(
      ( key, value ){        
        if( isEmpty(key) || value == null ) return;
        if( !( value is Map ) ) return;
        list.add(DataField(Paths.child(this.path, key.toString()), value));
      });
    return list;
  }
  FunctionsDocument._(
    {
      String path,
      Iterable<DataField> value,
      bool isTemporary = false,
      int group = 0,
      int order = 10
    }
  ) : super(
    path:path,
    children:value,
    isTemporary:isTemporary,
    group:group,
    order:order
  );

  /// Functions region.
  String get region {
    if( this._region != null ) return this._region;
    if( this.rawPath == null || isEmpty( this.rawPath.path ) ) return "asia-northeast1";
    RegExpMatch match = _regExp.firstMatch(this.rawPath.path);
    if( match != null ) return this._region = match.group(0);
    return null;
  }
  String _region;

  /// Get the protocol of the path.
  @override
  String get protocol {
    if( isEmpty( this.rawPath.scheme ) ) return "firestore";
    return this.rawPath.scheme;
  }

  /// Get the data.
  /// 
  /// Do not use from external class.
  /// 
  /// When using from an external class, use getInt or getString.
  /// 
  /// [key]: Key for retrieving data.
  @protected
  dynamic operator []( String key ){
    if( !this.data.containsKey( key ) ) return null;
    return this.data[key]?.data;
  }
  /// Set the data.
  /// 
  /// [key]: Key for storing data [value]: Data to store.
  void operator []=( String key, dynamic value ){
    if( value == null ) { return; }
    if( this.isLock ) { Log.error( "Data modification is prohibited." ); return; }
    if( value is DataField ){
      this.set( [value] );
    } else {
      if( PathFilter.setTemporary( Paths.child(this.path, key), value) ) return;
      if( this.data.containsKey( key ) && this.data[key] is IUnit<dynamic> ) {
         this.data[key].data = value;
      } else {
        Path path = this.rawPath.clone( );
        path.path = Paths.child( this.rawPath.path, key );
        this.set( [DataField( path.path, value )] );
      }
    }
  }

  /// Gets value as Bool.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  bool getBool(String key, [bool defaultValue = false]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getBool( defaultValue );
  }
  /// Gets value as Double.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  double getDouble(String key, [double defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getDouble( defaultValue );
  }
  /// Gets value as Int.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  int getInt(String key, [int defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getInt( defaultValue );
  }
  /// Gets value as List<T>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  List<T> getList<T extends Object>(
    String key,
    [
      List<T> defaultValue = const []
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getList<T>( defaultValue );
  }
  /// Gets value as Map<K,V>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  Map<K, V> getMap<K extends Object, V extends Object>(
    String key,
    [
      Map<K, V> defaultValue = const {}
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getMap<K,V>( defaultValue );
  }
  /// Gets value as String.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  String getString(String key, [String defaultValue = Const.empty]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getString( defaultValue );
  }


  /// Get the UID of the document.
  /// 
  /// If there is no value in the field, id will be output.
  String get uid => this.getString(Const.uid, this.id);
  /// Get time.
  /// UpdatedTime is output if the field has no value.
  int get time => this.getInt(Const.time, this.updatedTime);


  /// Save the data.
  /// 
  /// Run if you have a remote or need to save data.
  Future<T> save<T extends IDataDocument>( ) => Future<T>.delayed(Duration.zero);
  /// Delete the data.
  /// 
  /// Used when deleting data when there is a remote or when data needs to be saved.
  Future delete( ){
    if( this.isDisposed ) return Future.delayed(Duration.zero);
    this.dispose();
    return Future.delayed(Duration.zero);
  }

}

==> masamune_firebase-0.2.2.tar.gz: lib/functions/functionsdocument.dart <==
part of masamune.firebase;

/// Task class for executing Firebase Functions functions.
/// 
/// Execute the task with the [call()] method and wait for it to complete with await.
/// 
/// The function must be one layer of Map data.
/// 
/// Parse Map data and use it as a document.
class FunctionsDocument extends TaskDocument<DataField>
  implements ITask, IDataDocument<DataField> {
  static RegExp _regExp = RegExp( "^[a-zA-Z0-9]+-[a-zA-Z0-9]+" );
  /// Create a Completer that matches the class.
  /// 
  /// Do not use from external class.
  @override
  @protected
  Completer createCompleter() => Completer<FunctionsDocument>();
  /// Process to create a new instance.
  /// 
  /// Do not use from outside the class.
  /// 
  /// [path]: Destination path.
  /// [isTemporary]: True if the data is temporary.
  @override
  @protected
  T createInstance<T extends IClonable>( String path, bool isTemporary ) => FunctionsDocument._(
    path: path,
    isTemporary: isTemporary,
    group: this.group,
    order: this.order
  ) as T;
  Firebase get _app {
    if( this.__app == null ) this.__app = Firebase._find(this.protocol);
    return this.__app;
  }
  Firebase __app;
  FirestoreAuth get _auth {
    if( this.__auth == null ) this.__auth = FirestoreAuth._find(this.protocol);
    return this.__auth;
  }
  FirestoreAuth __auth;
  FirestoreFunctions get _functions {
    if( this.__functions == null ){
      this.__functions = FirestoreFunctions._find(this.protocol, this.region);
    }
    return this.__functions;
  }
  FirestoreFunctions __functions;
  HttpsCallable get _callable {
    if( this.__callable == null ){
      this.__callable = this._functions?._functions?.getHttpsCallable(
        functionName: this.rawPath.file
      );
    }
    return this.__callable;
  }
  HttpsCallable __callable;

  /// Task class for executing Firebase Functions functions.
  /// 
  /// Execute the task with the [call()] method and wait for it to complete with await.
  /// 
  /// The function must be one layer of Map data.
  /// 
  /// Parse Map data and use it as a document.
  /// 
  /// [path]: Specify the execution function of Functions in URL format (https://～).
  /// [defaultData]: Initial value when the function is not executed.
  /// [postData]: Parameters to pass to the function.
  /// [timeout]: Timeout time.
  static Future<FunctionsDocument> call(
    String path,
    {
      Map<String, dynamic> defaultValue,
      Map<String,dynamic> data = const {},
      Duration timeout = Const.timeout,
      OrderBy orderBy = OrderBy.none,
      OrderBy thenBy = OrderBy.none,
      String orderByKey,
      String thenByKey
    }
  ){
    path = path?.replaceAll("https", "firestore")?.applyTags();
    if( isEmpty(path) ) {
      Log.error( "Path is invalid." );
      return null;
    }
    FunctionsDocument document = PathMap.get<FunctionsDocument>( path );
    if( document != null ) {
      return document.recall(defaultValue: defaultValue, data:data, timeout:timeout);
    }
    document = FunctionsDocument._( path:path );
    if( defaultValue != null ){
      defaultValue?.forEach(
        (key, value) => document.setInternal( DataField( key, value ) )
      );
    }
    document._call(data, timeout);
    return document.future;
  }
  /// Run the function again.
  /// 
  /// [defaultData]: Initial value when the function is not executed.
  /// [postData]: Parameters to pass to the function.
  /// [timeout]: Timeout time.
  Future<FunctionsDocument> recall(
    {
      Map<String, dynamic> defaultValue,
      Map<String,dynamic> data = const {},
      Duration timeout = Const.timeout
    }
  ){
    this.init();
    if( defaultValue != null ){
      this.clear();
      if( defaultValue != null ){
        defaultValue?.forEach(
          (key, value) => this.setInternal( DataField( key, value ) )
        );
      }
    }
    this._call(data, timeout);
    return this.future;
  }
  void _call(Map<String,dynamic> data, Duration timeout) async {
    try {
      if( this._app == null ) this.__app = await Firebase.initialize();
      if( this._auth == null ) this.__auth = await FirestoreAuth.signIn(protocol:this.protocol);
      if( this._functions == null ){
        this.__functions = await FirestoreFunctions.configure(
          protocol: this.protocol,
          region: this.region
        );
      }  
      if( this._callable == null ){
        this.__callable = this._functions._functions
          .getHttpsCallable(functionName: this.rawPath.file);
      }
      Map<dynamic,dynamic> res;
      if( data == null || data.length <= 0 ){
        res = ( await this._callable.call( ).timeout(timeout) )?.data;
      } else {
        res = ( await this._callable.call( data ).timeout(timeout) )?.data;
      }
      if( res == null || res.length <= 0 ){
        this.done();
        return;
      }
      this._build(res)?.forEach((val) => this.setInternal( val ));
      this.done();
    } on TimeoutException catch(e){
      this.timeout(e.toString());
    } catch( e ) {
      this.error( e.toString() );
    }
  }
  Iterable<DataField> _build( Map<dynamic, dynamic> data ) {
    List<DataField> list = ListPool.get();
    data?.forEach(
      ( key, value ){        
        if( isEmpty(key) || value == null ) return;
        if( !( value is Map ) ) return;
        list.add(DataField(Paths.child(this.path, key.toString()), value));
      });
    return list;
  }
  FunctionsDocument._(
    {
      String path,
      Iterable<DataField> value,
      bool isTemporary = false,
      int group = 0,
      int order = 10
    }
  ) : super(
    path:path,
    children:value,
    isTemporary:isTemporary,
    group:group,
    order:order
  );

  /// Functions region.
  String get region {
    if( this._region != null ) return this._region;
    if( this.rawPath == null || isEmpty( this.rawPath.path ) ) return "asia-northeast1";
    RegExpMatch match = _regExp.firstMatch(this.rawPath.path);
    if( match != null ) return this._region = match.group(0);
    return null;
  }
  String _region;

  /// Get the protocol of the path.
  @override
  String get protocol {
    if( isEmpty( this.rawPath.scheme ) ) return "firestore";
    return this.rawPath.scheme;
  }

  /// Get the data.
  /// 
  /// Do not use from external class.
  /// 
  /// When using from an external class, use getInt or getString.
  /// 
  /// [key]: Key for retrieving data.
  @protected
  dynamic operator []( String key ){
    if( !this.data.containsKey( key ) ) return null;
    return this.data[key]?.data;
  }
  /// Set the data.
  /// 
  /// [key]: Key for storing data [value]: Data to store.
  void operator []=( String key, dynamic value ){
    if( value == null ) { return; }
    if( this.isLock ) { Log.error( "Data modification is prohibited." ); return; }
    if( value is DataField ){
      this.set( [value] );
    } else {
      if( PathFilter.setTemporary( Paths.child(this.path, key), value) ) return;
      if( this.data.containsKey( key ) && this.data[key] is IUnit<dynamic> ) {
         this.data[key].data = value;
      } else {
        Path path = this.rawPath.clone( );
        path.path = Paths.child( this.rawPath.path, key );
        this.set( [DataField( path.path, value )] );
      }
    }
  }

  /// Gets value as Bool.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  bool getBool(String key, [bool defaultValue = false]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getBool( defaultValue );
  }
  /// Gets value as Double.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  double getDouble(String key, [double defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getDouble( defaultValue );
  }
  /// Gets value as Int.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  int getInt(String key, [int defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getInt( defaultValue );
  }
  /// Gets value as List<T>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  List<T> getList<T extends Object>(
    String key,
    [
      List<T> defaultValue = const []
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getList<T>( defaultValue );
  }
  /// Gets value as Map<K,V>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  Map<K, V> getMap<K extends Object, V extends Object>(
    String key,
    [
      Map<K, V> defaultValue = const {}
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getMap<K,V>( defaultValue );
  }
  /// Gets value as String.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  String getString(String key, [String defaultValue = Const.empty]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getString( defaultValue );
  }


  /// Get the UID of the document.
  /// 
  /// If there is no value in the field, id will be output.
  String get uid => this.getString(Const.uid, this.id);
  /// Get time.
  /// UpdatedTime is output if the field has no value.
  int get time => this.getInt(Const.time, this.updatedTime);


  /// Save the data.
  /// 
  /// Run if you have a remote or need to save data.
  Future<T> save<T extends IDataDocument>( ) => Future<T>.delayed(Duration.zero);
  /// Delete the data.
  /// 
  /// Used when deleting data when there is a remote or when data needs to be saved.
  Future delete( ){
    if( this.isDisposed ) return Future.delayed(Duration.zero);
    this.dispose();
    return Future.delayed(Duration.zero);
  }

}
